# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:/bld/qgis_1616390310858/work/python/console/console_settings.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SettingsDialogPythonConsole(object):
    def setupUi(self, SettingsDialogPythonConsole):
        SettingsDialogPythonConsole.setObjectName("SettingsDialogPythonConsole")
        SettingsDialogPythonConsole.resize(1055, 731)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SettingsDialogPythonConsole.sizePolicy().hasHeightForWidth())
        SettingsDialogPythonConsole.setSizePolicy(sizePolicy)
        SettingsDialogPythonConsole.setMinimumSize(QtCore.QSize(500, 0))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        SettingsDialogPythonConsole.setFont(font)
        self.gridLayout_16 = QtWidgets.QGridLayout(SettingsDialogPythonConsole)
        self.gridLayout_16.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_16.setObjectName("gridLayout_16")
        self.scrollArea = QtWidgets.QScrollArea(SettingsDialogPythonConsole)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 1055, 731))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_4 = QgsCollapsibleGroupBox(self.scrollAreaWidgetContents)
        self.groupBox_4.setProperty("collapsed", True)
        self.groupBox_4.setProperty("saveCollapsedState", True)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.autoInsertionImport = QtWidgets.QCheckBox(self.groupBox_4)
        self.autoInsertionImport.setObjectName("autoInsertionImport")
        self.gridLayout_11.addWidget(self.autoInsertionImport, 1, 0, 1, 1)
        self.autoCloseBracket = QtWidgets.QCheckBox(self.groupBox_4)
        self.autoCloseBracket.setObjectName("autoCloseBracket")
        self.gridLayout_11.addWidget(self.autoCloseBracket, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_4, 1, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.tokenGhLineEdit = QtWidgets.QLineEdit(self.groupBox)
        self.tokenGhLineEdit.setObjectName("tokenGhLineEdit")
        self.gridLayout_6.addWidget(self.tokenGhLineEdit, 0, 0, 1, 1)
        self.generateToken = QtWidgets.QPushButton(self.groupBox)
        self.generateToken.setObjectName("generateToken")
        self.gridLayout_6.addWidget(self.generateToken, 0, 1, 1, 1)
        self.gridLayout.addWidget(self.groupBox, 5, 0, 1, 1)
        self.groupBoxAutoCompletion = QgsCollapsibleGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxAutoCompletion.setCheckable(True)
        self.groupBoxAutoCompletion.setChecked(True)
        self.groupBoxAutoCompletion.setProperty("collapsed", False)
        self.groupBoxAutoCompletion.setProperty("saveCheckedState", True)
        self.groupBoxAutoCompletion.setProperty("saveCollapsedState", True)
        self.groupBoxAutoCompletion.setObjectName("groupBoxAutoCompletion")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxAutoCompletion)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_36 = QtWidgets.QLabel(self.groupBoxAutoCompletion)
        self.label_36.setObjectName("label_36")
        self.gridLayout_5.addWidget(self.label_36, 0, 2, 1, 1)
        self.autoCompThreshold = QtWidgets.QSpinBox(self.groupBoxAutoCompletion)
        self.autoCompThreshold.setMaximum(10)
        self.autoCompThreshold.setProperty("value", 2)
        self.autoCompThreshold.setObjectName("autoCompThreshold")
        self.gridLayout_5.addWidget(self.autoCompThreshold, 0, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.groupBoxAutoCompletion)
        self.label_6.setObjectName("label_6")
        self.gridLayout_5.addWidget(self.label_6, 0, 0, 1, 1)
        self.layoutRadioButton = QtWidgets.QGridLayout()
        self.layoutRadioButton.setObjectName("layoutRadioButton")
        self.autoCompFromDocAPI = QtWidgets.QRadioButton(self.groupBoxAutoCompletion)
        self.autoCompFromDocAPI.setObjectName("autoCompFromDocAPI")
        self.layoutRadioButton.addWidget(self.autoCompFromDocAPI, 0, 2, 1, 1)
        self.autoCompFromAPI = QtWidgets.QRadioButton(self.groupBoxAutoCompletion)
        self.autoCompFromAPI.setChecked(True)
        self.autoCompFromAPI.setObjectName("autoCompFromAPI")
        self.layoutRadioButton.addWidget(self.autoCompFromAPI, 0, 1, 1, 1)
        self.autoCompFromDoc = QtWidgets.QRadioButton(self.groupBoxAutoCompletion)
        self.autoCompFromDoc.setObjectName("autoCompFromDoc")
        self.layoutRadioButton.addWidget(self.autoCompFromDoc, 0, 0, 1, 1)
        self.gridLayout_5.addLayout(self.layoutRadioButton, 1, 0, 1, 3)
        self.gridLayout.addWidget(self.groupBoxAutoCompletion, 0, 0, 1, 1)
        self.groupBox_2 = QgsCollapsibleGroupBox(self.scrollAreaWidgetContents)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.preloadAPI = QtWidgets.QCheckBox(self.groupBox_2)
        self.preloadAPI.setChecked(True)
        self.preloadAPI.setObjectName("preloadAPI")
        self.gridLayout_7.addWidget(self.preloadAPI, 0, 0, 1, 1)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.addAPIpath = QtWidgets.QPushButton(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addAPIpath.sizePolicy().hasHeightForWidth())
        self.addAPIpath.setSizePolicy(sizePolicy)
        self.addAPIpath.setMinimumSize(QtCore.QSize(32, 32))
        self.addAPIpath.setMaximumSize(QtCore.QSize(32, 32))
        self.addAPIpath.setText("")
        self.addAPIpath.setObjectName("addAPIpath")
        self.verticalLayout.addWidget(self.addAPIpath)
        self.removeAPIpath = QtWidgets.QPushButton(self.groupBox_2)
        self.removeAPIpath.setMinimumSize(QtCore.QSize(32, 32))
        self.removeAPIpath.setMaximumSize(QtCore.QSize(32, 32))
        self.removeAPIpath.setText("")
        self.removeAPIpath.setObjectName("removeAPIpath")
        self.verticalLayout.addWidget(self.removeAPIpath)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.gridLayout_2.addLayout(self.verticalLayout, 0, 1, 1, 1)
        self.tableWidget = QtWidgets.QTableWidget(self.groupBox_2)
        self.tableWidget.setMinimumSize(QtCore.QSize(0, 150))
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setTabKeyNavigation(False)
        self.tableWidget.setProperty("showDropIndicator", False)
        self.tableWidget.setDragDropOverwriteMode(False)
        self.tableWidget.setAlternatingRowColors(True)
        self.tableWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.tableWidget.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableWidget.setVerticalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
        self.tableWidget.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
        self.tableWidget.setGridStyle(QtCore.Qt.SolidLine)
        self.tableWidget.setRowCount(0)
        self.tableWidget.setColumnCount(2)
        self.tableWidget.setObjectName("tableWidget")
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        self.tableWidget.horizontalHeader().setVisible(False)
        self.tableWidget.horizontalHeader().setCascadingSectionResizes(False)
        self.tableWidget.horizontalHeader().setSortIndicatorShown(False)
        self.tableWidget.horizontalHeader().setStretchLastSection(True)
        self.tableWidget.verticalHeader().setVisible(False)
        self.gridLayout_2.addWidget(self.tableWidget, 0, 0, 1, 1)
        self.gridLayout_7.addLayout(self.gridLayout_2, 1, 0, 1, 1)
        self.groupBoxPreparedAPI = QgsCollapsibleGroupBox(self.groupBox_2)
        self.groupBoxPreparedAPI.setCheckable(True)
        self.groupBoxPreparedAPI.setChecked(False)
        self.groupBoxPreparedAPI.setProperty("collapsed", True)
        self.groupBoxPreparedAPI.setProperty("saveCollapsedState", True)
        self.groupBoxPreparedAPI.setProperty("saveCheckedState", True)
        self.groupBoxPreparedAPI.setObjectName("groupBoxPreparedAPI")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxPreparedAPI)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.compileAPIs = QtWidgets.QPushButton(self.groupBoxPreparedAPI)
        self.compileAPIs.setObjectName("compileAPIs")
        self.gridLayout_3.addWidget(self.compileAPIs, 0, 0, 1, 1)
        self.lineEdit = QtWidgets.QLineEdit(self.groupBoxPreparedAPI)
        self.lineEdit.setMinimumSize(QtCore.QSize(0, 27))
        self.lineEdit.setObjectName("lineEdit")
        self.gridLayout_3.addWidget(self.lineEdit, 0, 1, 1, 1)
        self.gridLayout_7.addWidget(self.groupBoxPreparedAPI, 2, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_2, 4, 0, 1, 1)
        self.groupBoxRunDebugEditor = QgsCollapsibleGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxRunDebugEditor.setProperty("collapsed", False)
        self.groupBoxRunDebugEditor.setProperty("saveCollapsedState", True)
        self.groupBoxRunDebugEditor.setObjectName("groupBoxRunDebugEditor")
        self.gridLayout_21 = QtWidgets.QGridLayout(self.groupBoxRunDebugEditor)
        self.gridLayout_21.setObjectName("gridLayout_21")
        self.enableObjectInspector = QtWidgets.QCheckBox(self.groupBoxRunDebugEditor)
        self.enableObjectInspector.setChecked(False)
        self.enableObjectInspector.setObjectName("enableObjectInspector")
        self.gridLayout_21.addWidget(self.enableObjectInspector, 0, 0, 1, 1)
        self.autoSaveScript = QtWidgets.QCheckBox(self.groupBoxRunDebugEditor)
        self.autoSaveScript.setObjectName("autoSaveScript")
        self.gridLayout_21.addWidget(self.autoSaveScript, 1, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBoxRunDebugEditor, 2, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout_16.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(SettingsDialogPythonConsole)
        QtCore.QMetaObject.connectSlotsByName(SettingsDialogPythonConsole)
        SettingsDialogPythonConsole.setTabOrder(self.scrollArea, self.groupBoxAutoCompletion)
        SettingsDialogPythonConsole.setTabOrder(self.groupBoxAutoCompletion, self.autoCompThreshold)
        SettingsDialogPythonConsole.setTabOrder(self.autoCompThreshold, self.autoCompFromDoc)
        SettingsDialogPythonConsole.setTabOrder(self.autoCompFromDoc, self.autoCompFromAPI)
        SettingsDialogPythonConsole.setTabOrder(self.autoCompFromAPI, self.autoCompFromDocAPI)
        SettingsDialogPythonConsole.setTabOrder(self.autoCompFromDocAPI, self.autoCloseBracket)
        SettingsDialogPythonConsole.setTabOrder(self.autoCloseBracket, self.autoInsertionImport)
        SettingsDialogPythonConsole.setTabOrder(self.autoInsertionImport, self.enableObjectInspector)
        SettingsDialogPythonConsole.setTabOrder(self.enableObjectInspector, self.autoSaveScript)
        SettingsDialogPythonConsole.setTabOrder(self.autoSaveScript, self.preloadAPI)
        SettingsDialogPythonConsole.setTabOrder(self.preloadAPI, self.tableWidget)
        SettingsDialogPythonConsole.setTabOrder(self.tableWidget, self.addAPIpath)
        SettingsDialogPythonConsole.setTabOrder(self.addAPIpath, self.removeAPIpath)
        SettingsDialogPythonConsole.setTabOrder(self.removeAPIpath, self.groupBoxPreparedAPI)
        SettingsDialogPythonConsole.setTabOrder(self.groupBoxPreparedAPI, self.compileAPIs)
        SettingsDialogPythonConsole.setTabOrder(self.compileAPIs, self.lineEdit)
        SettingsDialogPythonConsole.setTabOrder(self.lineEdit, self.tokenGhLineEdit)
        SettingsDialogPythonConsole.setTabOrder(self.tokenGhLineEdit, self.generateToken)

    def retranslateUi(self, SettingsDialogPythonConsole):
        _translate = QtCore.QCoreApplication.translate
        self.groupBox_4.setTitle(_translate("SettingsDialogPythonConsole", "Typing"))
        self.autoInsertionImport.setText(_translate("SettingsDialogPythonConsole", "Automatic insertion of the \'import\' string on \'from xxx\'"))
        self.autoCloseBracket.setText(_translate("SettingsDialogPythonConsole", "Automatic parentheses insertion"))
        self.groupBox.setTitle(_translate("SettingsDialogPythonConsole", "GitHub Access Token"))
        self.tokenGhLineEdit.setPlaceholderText(_translate("SettingsDialogPythonConsole", "<PERSONAL_ACCESS_TOKEN>"))
        self.generateToken.setText(_translate("SettingsDialogPythonConsole", "Generate"))
        self.groupBoxAutoCompletion.setTitle(_translate("SettingsDialogPythonConsole", "Autocompletion"))
        self.label_36.setText(_translate("SettingsDialogPythonConsole", "characters"))
        self.label_6.setText(_translate("SettingsDialogPythonConsole", "Autocompletion threshold"))
        self.autoCompFromDocAPI.setToolTip(_translate("SettingsDialogPythonConsole", "Get autocompletion from current document and installed APIs"))
        self.autoCompFromDocAPI.setText(_translate("SettingsDialogPythonConsole", "From doc and APIs"))
        self.autoCompFromAPI.setToolTip(_translate("SettingsDialogPythonConsole", "Get autocompletion from installed APIs"))
        self.autoCompFromAPI.setText(_translate("SettingsDialogPythonConsole", "From API files"))
        self.autoCompFromDoc.setToolTip(_translate("SettingsDialogPythonConsole", "Get autocompletion from current document"))
        self.autoCompFromDoc.setText(_translate("SettingsDialogPythonConsole", "From document"))
        self.groupBox_2.setTitle(_translate("SettingsDialogPythonConsole", "APIs"))
        self.preloadAPI.setText(_translate("SettingsDialogPythonConsole", "Using preloaded APIs file"))
        item = self.tableWidget.horizontalHeaderItem(0)
        item.setText(_translate("SettingsDialogPythonConsole", "APIs"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("SettingsDialogPythonConsole", "Path"))
        self.groupBoxPreparedAPI.setTitle(_translate("SettingsDialogPythonConsole", "Using prepared APIs file"))
        self.compileAPIs.setText(_translate("SettingsDialogPythonConsole", "Compile APIs…"))
        self.groupBoxRunDebugEditor.setTitle(_translate("SettingsDialogPythonConsole", "Run and Debug"))
        self.enableObjectInspector.setText(_translate("SettingsDialogPythonConsole", "Enable Object Inspector (switching between tabs may be slow)"))
        self.autoSaveScript.setText(_translate("SettingsDialogPythonConsole", "Auto-save script before running"))
from qgis.gui import QgsCollapsibleGroupBox
